package pt.up.dcc.vcandroid;

import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Rect;
import android.hardware.Camera;
import android.hardware.Camera.PreviewCallback;
import android.util.Log;
import android.view.SurfaceHolder;

public class PreviewAnalyzer implements PreviewCallback {
	// Variveis globais de apoio
	private String TAG = "PreviewAnalyzer";
	private SurfaceHolder mHolder;
	private Canvas canvas;
	private Paint pt;
	//
	private int[] matrix;
	int sizex = 0;
	int sizey = 0;

	// Construtor
	public PreviewAnalyzer(SurfaceHolder holder) {
		mHolder = holder;
		pt = new Paint();
		pt.setColor(Color.RED);
		pt.setStrokeWidth(10);
	}

	// Metodo para uso externo, usado para indicar o tamanho do frame que vamos receber da camera
	public void setPreviewSize(Camera.Size s) {
		// Aloca um buffer de 32-bits do tamanho da frame que vamos receber da camera
		matrix = new int[s.width * s.height];
		sizex = s.width;
		sizey = s.height;
	}

	// Metodo para uso externo, o qual vai receber o frame capturado pela camera
	public void onPreviewFrame(byte[] data, Camera camera) {
		// reservamos, so para este processo, o canvas onde vamos colocar o resultado do frame processado
		canvas = mHolder.lockCanvas(null);
		try {
			// verifica se o novo frame recebido tem o mesmo tamanho do buffer anteriormente alocado
			// YUV 420 length = (width*height*1.5)
			if (data.length == matrix.length * 1.5) {
				// se tiver o mesmo tamanho podemos processar o frame
				processFrame(data, canvas);
			} else {
				// se nao tiver o mesmo tamanho mosta um ecra negro com um X vermelho
				canvas.drawColor(Color.BLACK);
				canvas.drawLine(0, 0, canvas.getWidth(), canvas.getHeight(), pt);
				canvas.drawLine(canvas.getWidth(), 0, 0, canvas.getHeight(), pt);
			}

		} catch (Exception e) {
			Log.e(TAG, "ERROR: processing frame ", e);
		} finally {
			// libertamos o canvas
			mHolder.unlockCanvasAndPost(canvas);
			canvas = null;
		}
	}

	/**
	 * Processa o frame e desenha a resposta no canvas
	 * 
	 *  frame - imagem no formato YUV (YUV 420)
	 *  canvas - canvas onde se vai desenhar o resultado do processamento
	 * 
	 * */
	private void processFrame(byte[] frame, Canvas canvas) {

		// Exemplo: Converso de uma imagem a cores, para uma imagem a preto e branco

		// Variveis de apoio
		int verde, vermelho, azul, cinzento;
		int x;

		// Converter o frame que usa o espaco de cores YUV para RGB
		// Coloca o frame convertido na variavel  matrix
		decodeYUVtoRGB(matrix, frame, sizex, sizey);
		
		// Obter matriz da imagem
		// A varivel "matrix" fica com os valores de cada pixel da imagem
		// A dimenso desta  determinada por "sizex" e "sizey"
		// Cada valor tm 4 bytes. Estes correspondem invidividualmente a:
		// Transparncia, Vermelho, Verde, Azul
		// Para aceder aos valores individuais:
		//		red = (color >> 16) & 0xff;
		//	    green = (color >> 8) & 0xff;
		//		blue = color & 0xff;
		

		
		// Ciclo que percorre a imagem inteira
		for (x = 0; x < sizex * sizey; x++) {
			vermelho = getRed(matrix[x]);
			verde = getGreen(matrix[x]);
			azul = getBlue(matrix[x]);

			// Calcular luminosidade
			cinzento = (vermelho + verde + azul) / 3;

			// Criar valor de cor
			matrix[x] = makeColor(cinzento, cinzento, cinzento);

		}

		
		
		// Aps a manipulaao da matrix,  necessrio criar o objecto grfico para poder visualizar o resultado
		Rect dst = new Rect(0, 0, (sizex * canvas.getHeight()) / sizey, canvas.getHeight());
		Bitmap bmp = Bitmap.createBitmap(matrix, sizex, sizey, Bitmap.Config.ARGB_8888);
		// Desenhar a imagem para o canvas externo de visualizao
		canvas.drawColor(Color.BLACK);
		canvas.drawBitmap(bmp, null, dst, pt);

	}

	
	
	// Funes de apoio para extrair os valores de R, G e B de uma imagem.
	private int getRed(int color) {	return (color >> 16) & 0xff;}
	private int getGreen(int color) {return (color >> 8) & 0xff;}
	private int getBlue(int color) {return color & 0xff;}
	private int makeColor(int red, int green, int blue) {return (255 << 24) | (red << 16) | (green << 8) | blue;}

	// Funes de apoio para converter uma imagem YUV para RGB.
	static public void decodeYUVtoRGB(int[] rgb, byte[] yuv420sp, int width, int height) {
		final int frameSize = width * height;

		for (int j = 0, yp = 0; j < height; j++) {
			int uvp = frameSize + (j >> 1) * width, u = 0, v = 0;
			for (int i = 0; i < width; i++, yp++) {
				int y = (0xff & ((int) yuv420sp[yp])) - 16;
				if (y < 0)
					y = 0;
				if ((i & 1) == 0) {
					v = (0xff & yuv420sp[uvp++]) - 128;
					u = (0xff & yuv420sp[uvp++]) - 128;
				}
				int y1192 = 1192 * y;
				int r = (y1192 + 1634 * v);
				int g = (y1192 - 833 * v - 400 * u);
				int b = (y1192 + 2066 * u);
				if (r < 0)
					r = 0;
				else if (r > 262143)
					r = 262143;
				if (g < 0)
					g = 0;
				else if (g > 262143)
					g = 262143;
				if (b < 0)
					b = 0;
				else if (b > 262143)
					b = 262143;
				rgb[yp] = 0xff000000 | ((r << 6) & 0xff0000) | ((g >> 2) & 0xff00) | ((b >> 10) & 0xff);
			}
		}
	}
}
